using NUnit.Framework;
using System.Collections;
using System.Reflection;
using System.Text;
using TSLib;
using TSLib.Messages;

namespace TS3ABotUnitTests
{
	[TestFixture]
	public class TS3MessageParserTests
	{
		private readonly Deserializer deserializer = new Deserializer();

		[Test]
		public void Deserializer1Test()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("cid=6"), NotificationType.ChannelChanged);
			Assert.NotNull(notif);
			Assert.AreEqual(1, notif.Length);
			var notifs = notif[0];
			AssertEx.PropertyValuesAreEquals(notifs, new ChannelChanged() { ChannelId = (ChannelId)6 });
		}

		[Test]
		public void Deserializer2Test()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("clid=42 cluid=asdfe\\/rvt=="), NotificationType.ClientChatComposing);
			Assert.NotNull(notif);
			Assert.AreEqual(1, notif.Length);
			var notifs = notif[0];
			AssertEx.PropertyValuesAreEquals(notifs, new ClientChatComposing() { ClientId = (ClientId)42, ClientUid = (Uid)"asdfe/rvt==" });
		}

		[Test]
		public void Deserializer3Test()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("cid=5 | cid=4"), NotificationType.ChannelChanged);
			Assert.NotNull(notif);
			Assert.AreEqual(2, notif.Length);
			AssertEx.PropertyValuesAreEquals(notif[0], new ChannelChanged() { ChannelId = (ChannelId)5 });
			AssertEx.PropertyValuesAreEquals(notif[1], new ChannelChanged() { ChannelId = (ChannelId)4 });
		}

		[Test]
		public void Deserializer4Test()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("cluid=asdfe\\/rvt== clid=42 | clid=1337"), NotificationType.ClientChatComposing);
			Assert.NotNull(notif);
			Assert.AreEqual(2, notif.Length);
			AssertEx.PropertyValuesAreEquals(notif[0], new ClientChatComposing() { ClientId = (ClientId)42, ClientUid = (Uid)"asdfe/rvt==" });
			AssertEx.PropertyValuesAreEquals(notif[1], new ClientChatComposing() { ClientId = (ClientId)1337, ClientUid = (Uid)"asdfe/rvt==" });
		}

		[Test]
		public void Deserializer5Test()
		{
			var notif = deserializer.GenerateResponse<ClientList>(Encoding.UTF8.GetBytes(
				"clid=1 cid=1 client_database_id=2 client_nickname=TestBob1 client_type=0 client_unique_identifier=u\\/dFMOFFipxS9fJ8HKv0KH6WVzA="
				+ "|clid=2 cid=4 client_database_id=2 client_nickname=TestBob client_type=0 client_unique_identifier=u\\/dFMOFFipxS9fJ8HKv0KH6WVzA="
				+ "|clid=3 cid=4 client_database_id=6 client_nickname=Splamy client_type=0 client_unique_identifier=uA0U7t4PBxdJ5TLnarsOHQh4\\/tY="
				+ "|clid=4 cid=4 client_database_id=7 client_nickname=AudioBud client_type=0 client_unique_identifier=b+P0CqXms5I0C+A66HZ4Sbu\\/PNw="
			));
			Assert.NotNull(notif);

			Assert.AreEqual(4, notif.Length);
			AssertEx.PropertyValuesAreEquals(notif[0], new ClientList() { ClientId = (ClientId)1, ChannelId = (ChannelId)1, DatabaseId = (ClientDbId)2, Name = "TestBob1", ClientType = ClientType.Full, Uid = (Uid)"u/dFMOFFipxS9fJ8HKv0KH6WVzA=" });
			AssertEx.PropertyValuesAreEquals(notif[1], new ClientList() { ClientId = (ClientId)2, ChannelId = (ChannelId)4, DatabaseId = (ClientDbId)2, Name = "TestBob", ClientType = ClientType.Full, Uid = (Uid)"u/dFMOFFipxS9fJ8HKv0KH6WVzA=" });
			AssertEx.PropertyValuesAreEquals(notif[2], new ClientList() { ClientId = (ClientId)3, ChannelId = (ChannelId)4, DatabaseId = (ClientDbId)6, Name = "Splamy", ClientType = ClientType.Full, Uid = (Uid)"uA0U7t4PBxdJ5TLnarsOHQh4/tY=" });
			AssertEx.PropertyValuesAreEquals(notif[3], new ClientList() { ClientId = (ClientId)4, ChannelId = (ChannelId)4, DatabaseId = (ClientDbId)7, Name = "AudioBud", ClientType = ClientType.Full, Uid = (Uid)"b+P0CqXms5I0C+A66HZ4Sbu/PNw=" });
		}

		[Test]
		public void Deserializer6DOnlyTest()
		{
			var notif = deserializer.GenerateResponse<ResponseDictionary>(Encoding.UTF8.GetBytes("cmd a=1 c=3 b=2|b=4|b=5"));
			Assert.NotNull(notif);
			Assert.AreEqual(3, notif.Length);
		}

		[Test]
		public void Deserializer7DOnlyTest()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("clientinitiv alpha=41Te9Ar7hMPx+A== omega=MEwDAgcAAgEgAiEAq2iCMfcijKDZ5tn2tuZcH+\\/GF+dmdxlXjDSFXLPGadACIHzUnbsPQ0FDt34Su4UXF46VFI0+4wjMDNszdoDYocu0 ip"), NotificationType.ClientInitIv);
			Assert.NotNull(notif);
		}

		[Test]
		public void Deserializer8DOnlyTest()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("initserver virtualserver_name=Server\\sder\\sVerplanten virtualserver_welcomemessage=This\\sis\\sSplamys\\sWorld virtualserver_platform=Linux virtualserver_version=3.0.13.8\\s[Build:\\s1500452811] virtualserver_maxclients=32 virtualserver_created=0 virtualserver_nodec_encryption_mode=1 virtualserver_hostmessage=L\xc3\xa9\\sServer\\sde\\sSplamy virtualserver_name=Server_mode=0 virtualserver_default_server group=8 virtualserver_default_channel_group=8 virtualserver_hostbanner_url virtualserver_hostmessagegfx_url virtualserver_hostmessagegfx_interval=2000 virtualserver_priority_speaker_dimm_modificat"), NotificationType.InitServer);
			Assert.NotNull(notif);
		}

		[Test]
		public void Deserializer9DOnlyTest()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes("channellist cid=2 cpid=0 channel_name=Trusted\\sChannel channel_topic channel_codec=0 channel_codec_quality=0 channel_maxclients=0 channel_maxfamilyclients=-1 channel_order=1 channel_flag_permanent=1 channel_flag_semi_permanent=0 channel_flag_default=0 channel_flag_password=0 channel_codec_latency_factor=1 channel_codec_is_unencrypted=1 channel_delete_delay=0 channel_flag_maxclients_unlimited=0 channel_flag_maxfamilyclients_unlimited=0 channel_flag_maxfamilyclients_inherited=1 channel_needed_talk_power=0 channel_forced_silence=0 channel_name_phonetic channel_icon_id=0 channel_flag_private=0|cid=4 cpid=2 channel_name=Ding\\s•\\s1\\s\\p\\sSplamy´s\\sBett channel_topic channel_codec=4 channel_codec_quality=7 channel_maxclients=-1 channel_maxfamilyclients=-1 channel_order=0 channel_flag_permanent=1 channel_flag_semi_permanent=0 channel_flag_default=0 channel_flag_password=0 channel_codec_latency_factor=1 channel_codec_is_unencrypted=1 channel_delete_delay=0 channel_flag_maxclients_unlimited=1 channel_flag_maxfamilyclients_unlimited=0 channel_flag_maxfamilyclients_inherited=1 channel_needed_talk_power=0 channel_forced_silence=0 channel_name_phonetic=Neo\\sSeebi\\sEvangelion channel_icon_id=0 channel_flag_private=0"), NotificationType.ChannelList);
			Assert.NotNull(notif);
		}

		[Test]
		public void Deserializer10WeirdTest()
		{
			var notif = deserializer.GenerateNotification(Encoding.UTF8.GetBytes(@"notifypermissionlist group_id_end=0|group_id_end=7|group_id_end=13|group_id_end=18|group_id_end=21|group_id_end=21|group_id_end=33|group_id_end=47|group_id_end=77|group_id_end=82|group_id_end=83|group_id_end=106|group_id_end=126|group_id_end=132|group_id_end=143|group_id_end=151|group_id_end=160|group_id_end=162|group_id_end=170|group_id_end=172|group_id_end=190|group_id_end=197|group_id_end=215|group_id_end=227|group_id_end=232|group_id_end=248|permname=b_serverinstance_help_view permdesc=Retrieve\sinformation\sabout\sServerQuery\scommands|permname=b_serverinstance_version_view permdesc=Retrieve\sglobal\sserver\sversion\s(including\splatform\sand\sbuild\snumber)|permname=b_serverinstance_info_view permdesc=Retrieve\sglobal\sserver\sinformation|permname=b_serverinstance_virtualserver_list permdesc=List\svirtual\sservers\sstored\sin\sthe\sdatabase|permname=b_serverinstance_binding_list permdesc=List\sactive\sIP\sbindings\son\smulti-homed\smachines|permname=b_serverinstance_permission_list permdesc=List\spermissions\savailable\savailable\son\sthe\sserver\sinstance|permname=b_serverinstance_permission_find permdesc=Search\spermission\sassignments\sby\sname\sor\sID|permname=b_virtualserver_create permdesc=Create\svirtual\sservers|permname=b_virtualserver_delete permdesc=Delete\svirtual\sservers|permname=b_virtualserver_start_any permdesc=Start\sany\svirtual\sserver\sin\sthe\sserver\sinstance|permname=b_virtualserver_stop_any permdesc=Stop\sany\svirtual\sserver\sin\sthe\sserver\sinstance|permname=b_virtualserver_change_machine_id permdesc=Change\sa\svirtual\sservers\smachine\sID|permname=b_virtualserver_change_template permdesc=Edit\svirtual\sserver\sdefault\stemplate\svalues|permname=b_serverquery_login permdesc=Login\sto\sServerQuery|permname=b_serverinstance_textmessage_send permdesc=Send\stext\smessages\sto\sall\svirtual\sservers\sat\sonce|permname=b_serverinstance_log_view permdesc=Retrieve\sglobal\sserver\slog|permname=b_serverinstance_log_add permdesc=Write\sto\sglobal\sserver\slog|permname=b_serverinstance_stop permdesc=Shutdown\sthe\sserver\sprocess|permname=b_serverinstance_modify_settings permdesc=Edit\sglobal\ssettings|permname=b_serverinstance_modify_querygroup permdesc=Edit\sglobal\sServerQuery\sgroups|permname=b_serverinstance_modify_templates permdesc=Edit\sglobal\stemplate\sgroups|permname=b_virtualserver_select permdesc=Select\sa\svirtual\sserver|permname=b_virtualserver_info_view permdesc=Retrieve\svirtual\sserver\sinformation|permname=b_virtualserver_connectioninfo_view permdesc=Retrieve\svirtual\sserver\sconnection\sinformation|permname=b_virtualserver_channel_list permdesc=List\schannels\son\sa\svirtual\sserver|permname=b_virtualserver_channel_search permdesc=Search\sfor\schannels\son\sa\svirtual\sserver|permname=b_virtualserver_client_list permdesc=List\sclients\sonline\son\sa\svirtual\sserver|permname=b_virtualserver_client_search permdesc=Search\sfor\sclients\sonline\son\sa\svirtual\sserver|permname=b_virtualserver_client_dblist permdesc=List\sclient\sidentities\sknown\sby\sthe\svirtual\sserver|permname=b_virtualserver_client_dbsearch permdesc=Search\sfor\sclient\sidentities\sknown\sby\sthe\svirtual\sserver|permname=b_virtualserver_client_dbinfo permdesc=Retrieve\sclient\sinformation|permname=b_virtualserver_permission_find permdesc=Find\spermissions|permname=b_virtualserver_custom_search permdesc=Find\scustom\sfields|permname=b_virtualserver_start permdesc=Start\sown\svirtual\sserver|permname=b_virtualserver_stop permdesc=Stop\sown\svirtual\sserver|permname=b_virtualserver_token_list permdesc=List\sprivilege\skeys\savailable|permname=b_virtualserver_token_add permdesc=Create\snew\sprivilege\skeys|permname=b_virtualserver_token_use permdesc=Use\sa\sprivilege\skeys\sto\sgain\saccess\sto\sgroups|permname=b_virtualserver_token_delete permdesc=Delete\sa\sprivilege\skey|permname=b_virtualserver_log_view permdesc=Retrieve\svirtual\sserver\slog|permname=b_virtualserver_log_add permdesc=Write\sto\svirtual\sserver\slog|permname=b_virtualserver_join_ignore_password permdesc=Join\svirtual\sserver\signoring\sits\spassword|permname=b_virtualserver_notify_register permdesc=Register\sfor\sserver\snotifications|permname=b_virtualserver_notify_unregister permdesc=Unregister\sfrom\sserver\snotifications|permname=b_virtualserver_snapshot_create permdesc=Create\sserver\ssnapshots|permname=b_virtualserver_snapshot_deploy permdesc=Deploy\sserver\ssnapshots|permname=b_virtualserver_permission_reset permdesc=Reset\sthe\sserver\spermission\ssettings\sto\sdefault\svalues|permname=b_virtualserver_modify_name permdesc=Modify\sserver\sname|permname=b_virtualserver_modify_welcomemessage permdesc=Modify\swelcome\smessage|permname=b_virtualserver_modify_maxclients permdesc=Modify\sservers\smax\sclients|permname=b_virtualserver_modify_reserved_slots permdesc=Modify\sreserved\sslots|permname=b_virtualserver_modify_password permdesc=Modify\sserver\spassword|permname=b_virtualserver_modify_default_servergroup permdesc=Modify\sdefault\sServer\sGroup|permname=b_virtualserver_modify_default_channelgroup permdesc=Modify\sdefault\sChannel\sGroup|permname=b_virtualserver_modify_default_channeladmingroup permdesc=Modify\sdefault\sChannel\sAdmin\sGroup|permname=b_virtualserver_modify_channel_forced_silence permdesc=Modify\schannel\sforce\ssilence\svalue|permname=b_virtualserver_modify_complain permdesc=Modify\sindividual\scomplain\ssettings|permname=b_virtualserver_modify_antiflood permdesc=Modify\sindividual\santiflood\ssettings|permname=b_virtualserver_modify_ft_settings permdesc=Modify\sfile\stransfer\ssettings|permname=b_virtualserver_modify_ft_quotas permdesc=Modify\sfile\stransfer\squotas|permname=b_virtualserver_modify_hostmessage permdesc=Modify\sindividual\shostmessage\ssettings|permname=b_virtualserver_modify_hostbanner permdesc=Modify\sindividual\shostbanner\ssettings|permname=b_virtualserver_modify_hostbutton permdesc=Modify\sindividual\shostbutton\ssettings|permname=b_virtualserver_modify_port permdesc=Modify\sserver\sport|permname=b_virtualserver_modify_autostart permdesc=Modify\sserver\sautostart|permname=b_virtualserver_modify_needed_identity_security_level permdesc=Modify\srequired\sidentity\ssecurity\slevel|permname=b_virtualserver_modify_priority_speaker_dimm_modificator permdesc=Modify\spriority\sspeaker\sdimm\smodificator|permname=b_virtualserver_modify_log_settings permdesc=Modify\slog\ssettings|permname=b_virtualserver_modify_min_client_version permdesc=Modify\smin\sclient\sversion|permname=b_virtualserver_modify_icon_id permdesc=Modify\sserver\sicon|permname=b_virtualserver_modify_weblist permdesc=Modify\sweb\sserver\slist\sreporting\ssettings|permname=b_virtualserver_modify_codec_encryption_mode permdesc=Modify\scodec\sencryption\smode|permname=b_virtualserver_modify_temporary_passwords permdesc=Modify\stemporary\sserverpasswords|permname=b_virtualserver_modify_temporary_passwords_own permdesc=Modify\sown\stemporary\sserverpasswords|permname=b_virtualserver_modify_channel_temp_delete_delay_default permdesc=Modify\sdefault\stemporary\schannel\sdelete\sdelay|permname=b_virtualserver_modify_nickname permdesc=Modify\sserver\snicknames|permname=b_virtualserver_modify_integrations permdesc=Modify\sintegrations|permname=i_channel_min_depth permdesc=Min\schannel\screation\sdepth\sin\shierarchy|permname=i_channel_max_depth permdesc=Max\schannel\screation\sdepth\sin\shierarchy|permname=b_channel_group_inheritance_end permdesc=Stop\sinheritance\sof\schannel\sgroup\spermissions|permname=i_channel_permission_modify_power permdesc=Modify\schannel\spermission\spower|permname=i_channel_needed_permission_modify_power permdesc=Needed\smodify\schannel\spermission\spower|permname=b_channel_info_view permdesc=Retrieve\schannel\sinformation|permname=b_channel_create_child permdesc=Create\ssub-channels|permname=b_channel_create_permanent permdesc=Create\spermanent\schannels|permname=b_channel_create_semi_permanent permdesc=Create\ssemi-permanent\schannels|permname=b_channel_create_temporary permdesc=Create\stemporary\schannels|permname=b_channel_create_private permdesc=Create\sprivate\schannel|permname=b_channel_create_with_topic permdesc=Create\schannels\swith\sa\stopic|permname=b_channel_create_with_description permdesc=Create\schannels\swith\sa\sdescription|permname=b_channel_create_with_password permdesc=Create\spassword\sprotected\schannels|permname=b_channel_create_modify_with_codec_speex8 permdesc=Create\schannels\susing\sSpeex\sNarrowband\s(8\skHz)\scodecs|permname=b_channel_create_modify_with_codec_speex16 permdesc=Create\schannels\susing\sSpeex\sWideband\s(16\skHz)\scodecs|permname=b_channel_create_modify_with_codec_speex32 permdesc=Create\schannels\susing\sSpeex\sUltra-Wideband\s(32\skHz)\scodecs|permname=b_channel_create_modify_with_codec_celtmono48 permdesc=Create\schannels\susing\sthe\sCELT\sMono\s(48\skHz)\scodec|permname=b_channel_create_modify_with_codec_opusvoice permdesc=Create\schannels\susing\sOPUS\s(voice)\scodec|permname=b_channel_create_modify_with_codec_opusmusic permdesc=Create\schannels\susing\sOPUS\s(music)\scodec|permname=i_channel_create_modify_with_codec_maxquality permdesc=Create\schannels\swith\scustom\scodec\squality|permname=i_channel_create_modify_with_codec_latency_factor_min permdesc=Create\schannels\swith\sminimal\scustom\scodec\slatency\sfactor|permname=b_channel_create_with_maxclients permdesc=Create\schannels\swith\scustom\smax\sclients|permname=b_channel_create_with_maxfamilyclients permdesc=Create\schannels\swith\scustom\smax\sfamily\sclients|permname=b_channel_create_with_sortorder permdesc=Create\schannels\swith\scustom\ssort\sorder|permname=b_channel_create_with_default permdesc=Create\sdefault\schannels|permname=b_channel_create_with_needed_talk_power permdesc=Create\schannels\swith\sneeded\stalk\spower|permname=b_channel_create_modify_with_force_password permdesc=Create\snew\schannels\sonly\swith\spassword|permname=i_channel_create_modify_with_temp_delete_delay permdesc=Max\sdelete\sdelay\sfor\stemporary\schannels|permname=b_channel_modify_parent permdesc=Move\schannels|permname=b_channel_modify_make_default permdesc=Make\schannel\sdefault|permname=b_channel_modify_make_permanent permdesc=Make\schannel\spermanent|permname=b_channel_modify_make_semi_permanent permdesc=Make\schannel\ssemi-permanent|permname=b_channel_modify_make_temporary permdesc=Make\schannel\stemporary|permname=b_channel_modify_name permdesc=Modify\schannel\sname|permname=b_channel_modify_topic permdesc=Modify\schannel\stopic|permname=b_channel_modify_description permdesc=Modify\schannel\sdescription|permname=b_channel_modify_password permdesc=Modify\schannel\spassword|permname=b_channel_modify_codec permdesc=Modify\schannel\scodec|permname=b_channel_modify_codec_quality permdesc=Modify\schannel\scodec\squality|permname=b_channel_modify_codec_latency_factor permdesc=Modify\schannel\scodec\slatency\sfactor|permname=b_channel_modify_maxclients permdesc=Modify\schannels\smax\sclients|permname=b_channel_modify_maxfamilyclients permdesc=Modify\schannels\smax\sfamily\sclients|permname=b_channel_modify_sortorder permdesc=Modify\schannel\ssort\sorder|permname=b_channel_modify_needed_talk_power permdesc=Change\sneeded\schannel\stalk\spower|permname=i_channel_modify_power permdesc=Channel\smodify\spower|permname=i_channel_needed_modify_power permdesc=Needed\schannel\smodify\spower|permname=b_channel_modify_make_codec_encrypted permdesc=Make\schannel\scodec\sencrypted|permname=b_channel_modify_temp_delete_delay permdesc=Modify\stemporary\schannel\sdelete\sdelay|permname=b_channel_delete_permanent permdesc=Delete\spermanent\schannels|permname=b_channel_delete_semi_permanent permdesc=Delete\ssemi-permanent\schannels|permname=b_channel_delete_temporary permdesc=Delete\stemporary\schannels|permname=b_channel_delete_flag_force permdesc=Force\schannel\sdelete|permname=i_channel_delete_power permdesc=Delete\schannel\spower|permname=i_channel_needed_delete_power permdesc=Needed\sdelete\schannel\spower|permname=b_channel_join_permanent permdesc=Join\spermanent\schannels|permname=b_channel_join_semi_permanent permdesc=Join\ssemi-permanent\schannels|permname=b_channel_join_temporary permdesc=Join\stemporary\schannels|permname=b_channel_join_ignore_password permdesc=Join\schannel\signoring\sits\spassword|permname=b_channel_join_ignore_maxclients permdesc=Ignore\schannels\smax\sclients\slimit|permname=i_channel_join_power permdesc=Channel\sjoin\spower|permname=i_channel_needed_join_power permdesc=Needed\schannel\sjoin\spower|permname=i_channel_subscribe_power permdesc=Channel\ssubscribe\spower|permname=i_channel_needed_subscribe_power permdesc=Needed\schannel\ssubscribe\spower|permname=i_channel_description_view_power permdesc=Channel\sdescription\sview\spower|permname=i_channel_needed_description_view_power permdesc=Needed\schannel\sneeded\sdescription\sview\spower|permname=i_icon_id permdesc=Group\sicon\sidentifier|permname=i_max_icon_filesize permdesc=Max\sicon\sfilesize\sin\sbytes|permname=b_icon_manage permdesc=Enables\sicon\smanagement|permname=b_group_is_permanent permdesc=Group\sis\spermanent|permname=i_group_auto_update_type permdesc=Group\sauto-update\stype|permname=i_group_auto_update_max_value permdesc=Group\sauto-update\smax\svalue|permname=i_group_sort_id permdesc=Group\ssort\sid|permname=i_group_show_name_in_tree permdesc=Show\sgroup\sname\sin\stree\sdepending\son\sselected\smode|permname=b_virtualserver_servergroup_list permdesc=List\sserver\sgroups|permname=b_virtualserver_servergroup_permission_list permdesc=List\sserver\sgroup\spermissions|permname=b_virtualserver_servergroup_client_list permdesc=List\sclients\sfrom\sa\sserver\sgroup|permname=b_virtualserver_channelgroup_list permdesc=List\schannel\sgroups|permname=b_virtualserver_channelgroup_permission_list permdesc=List\schannel\sgroup\spermissions|permname=b_virtualserver_channelgroup_client_list permdesc=List\sclients\sfrom\sa\schannel\sgroup|permname=b_virtualserver_client_permission_list permdesc=List\sclient\spermissions|permname=b_virtualserver_channel_permission_list permdesc=List\schannel\spermissions|permname=b_virtualserver_channelclient_permission_list permdesc=List\schannel\sclient\spermissions|permname=b_virtualserver_servergroup_create permdesc=Create\sserver\sgroups|permname=b_virtualserver_channelgroup_create permdesc=Create\schannel\sgroups|permname=i_group_modify_power permdesc=Group\smodify\spower|permname=i_group_needed_modify_power permdesc=Needed\sgroup\smodify\spower|permname=i_group_member_add_power permdesc=Group\smember\sadd\spower|permname=i_group_needed_member_add_power permdesc=Needed\sgroup\smember\sadd\spower|permname=i_group_member_remove_power permdesc=Group\smember\sdelete\spower|permname=i_group_needed_member_remove_power permdesc=Needed\sgroup\smember\sdelete\spower|permname=i_permission_modify_power permdesc=Permission\smodify\spower|permname=b_permission_modify_power_ignore permdesc=Ignore\sneeded\spermission\smodify\spower|permname=b_virtualserver_servergroup_delete permdesc=Delete\sserver\sgroups|permname=b_virtualserver_channelgroup_delete permdesc=Delete\schannel\sgroups|permname=i_client_permission_modify_power permdesc=Client\spermission\smodify\spower|permname=i_client_needed_permission_modify_power permdesc=Needed\sclient\spermission\smodify\spower|permname=i_client_max_clones_uid permdesc=Max\sadditional\sconnections\sper\sclient\sidentity|permname=i_client_max_idletime permdesc=Max\sidle\stime\sin\sseconds|permname=i_client_max_avatar_filesize permdesc=Max\savatar\sfilesize\sin\sbytes|permname=i_client_max_channel_subscriptions permdesc=Max\schannel\ssubscriptions|permname=b_client_is_priority_speaker permdesc=Client\sis\spriority\sspeaker|permname=b_client_skip_channelgroup_permissions permdesc=Ignore\schannel\sgroup\spermissions|permname=b_client_force_push_to_talk permdesc=Force\sPush-To-Talk\scapture\smode|permname=b_client_ignore_bans permdesc=Ignore\sbans|permname=b_client_ignore_antiflood permdesc=Ignore\santiflood\smeasurements|permname=b_client_issue_client_query_command permdesc=Issue\squery\scommands\sfrom\sclient|permname=b_client_use_reserved_slot permdesc=Use\san\sreserved\sslot|permname=b_client_use_channel_commander permdesc=Use\schannel\scommander|permname=b_client_request_talker permdesc=Allow\sto\srequest\stalk\spower|permname=b_client_avatar_delete_other permdesc=Allow\sdeletion\sof\savatars\sfrom\sother\sclients|permname=b_client_is_sticky permdesc=Client\swill\sbe\ssticked\sto\scurrent\schannel|permname=b_client_ignore_sticky permdesc=Client\signores\ssticky\sflag|permname=b_client_info_view permdesc=Retrieve\sclient\sinformation|permname=b_client_permissionoverview_view permdesc=Retrieve\sclient\spermissions\soverview|permname=b_client_permissionoverview_own permdesc=Retrieve\sclients\sown\spermissions\soverview|permname=b_client_remoteaddress_view permdesc=View\sclient\sIP\saddress\sand\sport|permname=i_client_serverquery_view_power permdesc=ServerQuery\sview\spower|permname=i_client_needed_serverquery_view_power permdesc=Needed\sServerQuery\sview\spower|permname=b_client_custom_info_view permdesc=View\scustom\sfields|permname=i_client_kick_from_server_power permdesc=Client\skick\spower\sfrom\sserver|permname=i_client_needed_kick_from_server_power permdesc=Needed\sclient\skick\spower\sfrom\sserver|permname=i_client_kick_from_channel_power permdesc=Client\skick\spower\sfrom\schannel|permname=i_client_needed_kick_from_channel_power permdesc=Needed\sclient\skick\spower\sfrom\schannel|permname=i_client_ban_power permdesc=Client\sban\spower|permname=i_client_needed_ban_power permdesc=Needed\sclient\sban\spower|permname=i_client_move_power permdesc=Client\smove\spower|permname=i_client_needed_move_power permdesc=Needed\sclient\smove\spower|permname=i_client_complain_power permdesc=Complain\spower|permname=i_client_needed_complain_power permdesc=Needed\scomplain\spower|permname=b_client_complain_list permdesc=Show\scomplain\slist|permname=b_client_complain_delete_own permdesc=Delete\sown\scomplains|permname=b_client_complain_delete permdesc=Delete\scomplains|permname=b_client_ban_list permdesc=Show\sbanlist|permname=b_client_ban_create permdesc=Add\sa\sban|permname=b_client_ban_delete_own permdesc=Delete\sown\sbans|permname=b_client_ban_delete permdesc=Delete\sbans|permname=i_client_ban_max_bantime permdesc=Max\sbantime|permname=i_client_private_textmessage_power permdesc=Client\sprivate\smessage\spower|permname=i_client_needed_private_textmessage_power permdesc=Needed\sclient\sprivate\smessage\spower|permname=b_client_server_textmessage_send permdesc=Send\stext\smessages\sto\svirtual\sserver|permname=b_client_channel_textmessage_send permdesc=Send\stext\smessages\sto\schannel|permname=b_client_offline_textmessage_send permdesc=Send\soffline\smessages\sto\sclients|permname=i_client_talk_power permdesc=Client\stalk\spower|permname=i_client_needed_talk_power permdesc=Needed\sclient\stalk\spower|permname=i_client_poke_power permdesc=Client\spoke\spower|permname=i_client_needed_poke_power permdesc=Needed\sclient\spoke\spower|permname=b_client_set_flag_talker permdesc=Set\sthe\stalker\sflag\sfor\sclients\sand\sallow\sthem\sto\sspeak|permname=i_client_whisper_power permdesc=Client\swhisper\spower|permname=i_client_needed_whisper_power permdesc=Client\sneeded\swhisper\spower|permname=b_client_modify_description permdesc=Edit\sa\sclients\sdescription|permname=b_client_modify_own_description permdesc=Allow\sclient\sto\sedit\sown\sdescription|permname=b_client_modify_dbproperties permdesc=Edit\sa\sclients\sproperties\sin\sthe\sdatabase|permname=b_client_delete_dbproperties permdesc=Delete\sa\sclients\sproperties\sin\sthe\sdatabase|permname=b_client_create_modify_serverquery_login permdesc=Create\sor\smodify\sown\sServerQuery\saccount|permname=b_ft_ignore_password permdesc=Browse\sfiles\swithout\schannel\spassword|permname=b_ft_transfer_list permdesc=Retrieve\slist\sof\srunning\sfiletransfers|permname=i_ft_file_upload_power permdesc=File\supload\spower|permname=i_ft_needed_file_upload_power permdesc=Needed\sfile\supload\spower|permname=i_ft_file_download_power permdesc=File\sdownload\spower|permname=i_ft_needed_file_download_power permdesc=Needed\sfile\sdownload\spower|permname=i_ft_file_delete_power permdesc=File\sdelete\spower|permname=i_ft_needed_file_delete_power permdesc=Needed\sfile\sdelete\spower|permname=i_ft_file_rename_power permdesc=File\srename\spower|permname=i_ft_needed_file_rename_power permdesc=Needed\sfile\srename\spower|permname=i_ft_file_browse_power permdesc=File\sbrowse\spower|permname=i_ft_needed_file_browse_power permdesc=Needed\sfile\sbrowse\spower|permname=i_ft_directory_create_power permdesc=Create\sdirectory\spower|permname=i_ft_needed_directory_create_power permdesc=Needed\screate\sdirectory\spower|permname=i_ft_quota_mb_download_per_client permdesc=Download\squota\sper\sclient\sin\sMByte|permname=i_ft_quota_mb_upload_per_client permdesc=Upload\squota\sper\sclient\sin\sMByte"), NotificationType.PermList);
			Assert.NotNull(notif);
		}
	}

	public static class AssertEx
	{
		public static void PropertyValuesAreEquals(object actual, object expected)
		{
			foreach (PropertyInfo property in expected.GetType().GetProperties())
			{
				object expectedValue = property.GetValue(expected, null);
				object actualValue = property.GetValue(actual, null);

				if (actualValue is IList lists)
					AssertListsAreEquals(property, lists, (IList)expectedValue);
				else if (!Equals(expectedValue, actualValue))
					Assert.Fail("Property {0}.{1} does not match. Expected: {2} but was: {3}", property.DeclaringType.Name, property.Name, expectedValue, actualValue);
			}
		}

		private static void AssertListsAreEquals(PropertyInfo property, IList actualList, IList expectedList)
		{
			if (actualList.Count != expectedList.Count)
				Assert.Fail("Property {0}.{1} does not match. Expected IList containing {2} elements but was IList containing {3} elements", property.PropertyType.Name, property.Name, expectedList.Count, actualList.Count);

			for (int i = 0; i < actualList.Count; i++)
				if (!Equals(actualList[i], expectedList[i]))
					Assert.Fail("Property {0}.{1} does not match. Expected IList with element {1} equals to {2} but was IList with element {1} equals to {3}", property.PropertyType.Name, property.Name, expectedList[i], actualList[i]);
		}
	}
}
